"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var testing_1 = require('@angular/core/testing');
var testing_2 = require('@angular/compiler/testing');
var core_1 = require('@angular/core');
var platform_browser_1 = require('@angular/platform-browser');
var testing_3 = require('@angular/router/testing');
var content_component_1 = require('./content.component');
var sidenav_service_1 = require("./sidenav.service");
var schema_service_1 = require("../services/schema/schema.service");
var BehaviorSubject_1 = require('rxjs/BehaviorSubject');
testing_1.describe('Component: Content', function () {
    var builder;
    function setup(emptyModelsList) {
        if (emptyModelsList === void 0) { emptyModelsList = false; }
        return builder.createAsync(ComponentTestController)
            .then(function (fixture) {
            var debug = fixture.debugElement.query(platform_browser_1.By.css('app-content'));
            var comp = debug.componentInstance;
            fixture.detectChanges();
            if (true === emptyModelsList) {
                comp.models = [];
            }
            return {
                fixture: fixture,
                comp: comp,
                debugElement: debug
            };
        });
    }
    testing_1.beforeEachProviders(function () { return [
        content_component_1.ContentComponent,
        sidenav_service_1.SidenavService,
        schema_service_1.SchemaService,
        testing_3.ROUTER_FAKE_PROVIDERS
    ]; });
    testing_1.beforeEach(testing_1.inject([testing_2.TestComponentBuilder], function (tcb) {
        builder = tcb;
    }));
    testing_1.describe('default', function () {
        testing_1.it('should inject the component', testing_1.inject([content_component_1.ContentComponent], function (component) {
            testing_1.expect(component).toBeTruthy();
        }));
        testing_1.it('should create the component', testing_1.inject([], function () {
            return setup().then(function (api) {
                testing_1.expect(api.debugElement).toBeTruthy();
                testing_1.expect(api.comp).toBeTruthy();
            });
        }));
        testing_1.it('should have default model list', testing_1.inject([], function () {
            return setup().then(function (api) {
                testing_1.expect(api.comp.models.length).toEqual(1);
                testing_1.expect(api.comp.models).toContain({ title: 'Home', link: ['/home'] });
            });
        }));
    });
    testing_1.describe('initSidenav', function () {
        testing_1.it('should subscribe to sidenav onChange event', testing_1.inject([sidenav_service_1.SidenavService], function (sidenavService) {
            return setup().then(function (api) {
                spyOn(api.comp.sidenav, 'toggle');
                sidenavService.onChange.emit({ name: 'toggle' });
                testing_1.expect(api.comp.sidenav.toggle).toHaveBeenCalled();
            });
        }));
    });
    testing_1.describe('initModels', function () {
        testing_1.it('should reject wrong schema', testing_1.inject([schema_service_1.SchemaService], function (schemaService) {
            return setup(true).then(function (api) {
                schemaService.schema = new BehaviorSubject_1.BehaviorSubject(null);
                api.comp.initModels();
                testing_1.expect(api.comp.models.length).toEqual(0);
            });
        }));
        testing_1.it('should accept new models from schema', testing_1.inject([schema_service_1.SchemaService], function (schemaService) {
            return setup(true).then(function (api) {
                schemaService.schema = new BehaviorSubject_1.BehaviorSubject({
                    models: [{ title: 'Person', link: ['/people'] }]
                });
                api.comp.initModels();
                testing_1.expect(api.comp.models.length).toEqual(1);
                testing_1.expect(api.comp.models).toContain({ title: 'Person', link: ['/people'] });
            });
        }));
    });
    testing_1.describe('closeSidenav', function () {
        testing_1.it('should ask for sidenav to close', testing_1.inject([sidenav_service_1.SidenavService], function (sidenavService) {
            return setup().then(function (api) {
                spyOn(sidenavService, 'change');
                api.comp.closeSidenav();
                testing_1.expect(sidenavService.change).toHaveBeenCalled();
                testing_1.expect(sidenavService.change).toHaveBeenCalledWith('close');
            });
        }));
    });
    testing_1.describe('ngOnInit', function () {
        testing_1.it('should init sidenav', testing_1.inject([], function () {
            return setup().then(function (api) {
                spyOn(api.comp, 'initSidenav');
                api.comp.ngOnInit();
                testing_1.expect(api.comp.initSidenav).toHaveBeenCalled();
            });
        }));
        testing_1.it('should init models', testing_1.inject([], function () {
            return setup().then(function (api) {
                spyOn(api.comp, 'initModels');
                api.comp.ngOnInit();
                testing_1.expect(api.comp.initModels).toHaveBeenCalled();
            });
        }));
    });
    testing_1.describe('ngOnDestroy', function () {
        testing_1.it('should release sidenav listener', testing_1.inject([sidenav_service_1.SidenavService], function (sidenavService) {
            return setup().then(function (api) {
                spyOn(sidenavService.onChange, 'unsubscribe');
                api.comp.ngOnDestroy();
                testing_1.expect(sidenavService.onChange.unsubscribe).toHaveBeenCalled();
            });
        }));
        testing_1.it('should release schema listener', testing_1.inject([], function () {
            return setup().then(function (api) {
                spyOn(api.comp.modelsObserver, 'unsubscribe');
                api.comp.ngOnDestroy();
                testing_1.expect(api.comp.modelsObserver.unsubscribe).toHaveBeenCalled();
            });
        }));
    });
});
var ComponentTestController = (function () {
    function ComponentTestController() {
    }
    ComponentTestController = __decorate([
        core_1.Component({
            selector: 'test',
            template: "\n    <app-content></app-content>\n  ",
            directives: [content_component_1.ContentComponent]
        }), 
        __metadata('design:paramtypes', [])
    ], ComponentTestController);
    return ComponentTestController;
}());
//# sourceMappingURL=content.component.spec.js.map