"use strict";
var testing_1 = require('@angular/core/testing');
var entrypointHelper_1 = require('./entrypointHelper');
var EntryPoint_1 = require('../services/models/EntryPoint');
testing_1.describe('Entrypoint Helper', function () {
    testing_1.describe('.filterEntryPoints', function () {
        testing_1.it('should omit @context entry', function () {
            var results = entrypointHelper_1.entrypointHelper.filterEntryPoints({
                '@context': '/contexts/Entrypoint',
                'person': '/people'
            });
            testing_1.expect(Object.keys(results).length).toBe(1);
            testing_1.expect(results.person).toBeDefined();
        });
        testing_1.it('should omit @id entry', function () {
            var results = entrypointHelper_1.entrypointHelper.filterEntryPoints({
                '@id': '/',
                'person': '/people'
            });
            testing_1.expect(Object.keys(results).length).toBe(1);
            testing_1.expect(results.person).toBeDefined();
        });
        testing_1.it('should omit @type entry', function () {
            var results = entrypointHelper_1.entrypointHelper.filterEntryPoints({
                '@type': 'Entrypoint',
                'person': '/people'
            });
            testing_1.expect(Object.keys(results).length).toBe(1);
            testing_1.expect(results.person).toBeDefined();
        });
    });
    testing_1.describe('.filterSupportedClass', function () {
        testing_1.it('should omit #ConstraintViolation entry', function () {
            var results = entrypointHelper_1.entrypointHelper.filterSupportedClass([
                { '@id': '#ConstraintViolation' },
                { '@id': 'http://schema.org/Person' }
            ]);
            testing_1.expect(Object.keys(results).length).toBe(1);
            testing_1.expect(results[0]['@id']).toBe('http://schema.org/Person');
        });
        testing_1.it('should omit #ConstraintViolationList entry', function () {
            var results = entrypointHelper_1.entrypointHelper.filterSupportedClass([
                { '@id': '#ConstraintViolationList' },
                { '@id': 'http://schema.org/Person' }
            ]);
            testing_1.expect(Object.keys(results).length).toBe(1);
            testing_1.expect(results[0]['@id']).toBe('http://schema.org/Person');
        });
    });
    testing_1.describe('.getEntryPointByModel', function () {
        var entrypoints;
        testing_1.beforeEach(function () {
            entrypoints = [
                new EntryPoint_1.EntryPoint('Person'),
                new EntryPoint_1.EntryPoint('BlogPosting')
            ];
        });
        testing_1.it('should find an entrypoint by the model name', function () {
            var results = entrypointHelper_1.entrypointHelper.getEntryPointByModel('Person', entrypoints);
            testing_1.expect(results).toBeDefined();
            testing_1.expect(results instanceof EntryPoint_1.EntryPoint).toBeTruthy();
            testing_1.expect(results.model).toBe('Person');
        });
        testing_1.it('should notify if nothing is found', function () {
            var results = entrypointHelper_1.entrypointHelper.getEntryPointByModel('Customer', entrypoints);
            testing_1.expect(results).toBeUndefined();
        });
    });
    testing_1.describe('.getModelById', function () {
        testing_1.it('should find model name in URI', function () {
            testing_1.expect(entrypointHelper_1.entrypointHelper.getModelById('http://schema.org/Person')).toBe('Person');
            testing_1.expect(entrypointHelper_1.entrypointHelper.getModelById('http://schema.org/api/Person')).toBe('Person');
            testing_1.expect(entrypointHelper_1.entrypointHelper.getModelById('/Person')).toBe('Person');
        });
        testing_1.it('should let string unchanged if the name can\'t be extracted', function () {
            testing_1.expect(entrypointHelper_1.entrypointHelper.getModelById('Person')).toBe('Person');
        });
    });
    testing_1.describe('.getOperations', function () {
        testing_1.it('should get operations', function () {
            var operations = [
                { 'hydra:method': 'GET' },
                { 'hydra:method': 'POST' }
            ], results = entrypointHelper_1.entrypointHelper.getOperations(operations, {
                'method': 'hydra:method',
                'PagedCollection': 'hydra:PagedCollection'
            });
            testing_1.expect(results.length).toBe(2);
            testing_1.expect(results[0]).toContain('GET');
            testing_1.expect(results[1]).toContain('POST');
        });
        testing_1.it('should detect get collection operation', function () {
            var operations = [
                { 'hydra:method': 'GET', 'returns': 'hydra:PagedCollection' }
            ], results = entrypointHelper_1.entrypointHelper.getOperations(operations, {
                'method': 'hydra:method',
                'PagedCollection': 'hydra:PagedCollection'
            });
            testing_1.expect(results.length).toBe(1);
            testing_1.expect(results[0]).toContain(entrypointHelper_1.COLLECTION_METHOD);
        });
    });
});
//# sourceMappingURL=entrypointHelper.spec.js.map