"use strict";
var testing_1 = require('@angular/core/testing');
var schemaHelper_1 = require('./schemaHelper');
var Model_1 = require('../services/models/Model');
testing_1.describe('Schema title Helper', function () {
    testing_1.describe('.getSchemaTitle', function () {
        testing_1.it('should get title from hydra format title', function () {
            testing_1.expect(schemaHelper_1.schemaHelper.getSchemaTitle({ 'hydra:title': 'My first API' }, { 'title': 'hydra:title' })).toBe('My first API');
        });
        testing_1.it('should not get title from other format title', function () {
            testing_1.expect(schemaHelper_1.schemaHelper.getSchemaTitle({ 'title': 'My first API' }, { 'title': 'hydra:title' })).toBeUndefined();
        });
    });
    testing_1.describe('.populateModels', function () {
        testing_1.beforeEach(function () {
            spyOn(schemaHelper_1.schemaHelper, 'populateModel').and.returnValue(new Model_1.Model());
            spyOn(schemaHelper_1.schemaHelper, 'cleanModelsList').and.callFake(function (datas) { return datas; });
        });
        testing_1.it('should populate with model', function () {
            var results = schemaHelper_1.schemaHelper.populateModels({ 'hydra:supportedClass': ['Person', 'Blog Posting'] }, { 'supportedClass': 'hydra:supportedClass' });
            testing_1.expect(schemaHelper_1.schemaHelper.populateModel).toHaveBeenCalled();
            testing_1.expect(Array.isArray(results)).toBeTruthy();
            testing_1.expect(results.length).toBe(2);
            testing_1.expect(results[0] instanceof Model_1.Model).toBeTruthy();
        });
    });
    testing_1.describe('.populateModel', function () {
        testing_1.it('should return a model', function () {
            testing_1.expect(schemaHelper_1.schemaHelper.populateModel() instanceof Model_1.Model).toBeTruthy();
        });
    });
    testing_1.describe('.cleanModelsList', function () {
        testing_1.it('should reject non array parameter', function () {
            testing_1.expect(schemaHelper_1.schemaHelper.cleanModelsList(undefined)).toBe(null);
        });
        testing_1.it('should remove the last three entry of the array', function () {
            var jsonApiPlatform = [{
                    'hydra:supportedClass': [
                        {
                            '@id': 'http://schema.org/Person',
                            '@type': 'hydra:Class',
                            'rdfs:label': 'Person',
                            'hydra:title': 'Person',
                            'hydra:description': 'A person (alive, dead, undead, or fictional).',
                            'hydra:supportedProperty': [
                                {
                                    '@type': 'hydra:SupportedProperty',
                                    'hydra:property': {
                                        '@id': 'https://schema.org/name',
                                        '@type': 'rdf:Property',
                                        'rdfs:label': 'name',
                                        'domain': 'http://schema.org/Person',
                                        'range': 'xmls:string'
                                    },
                                    'hydra:title': 'name',
                                    'hydra:required': false,
                                    'hydra:readable': true,
                                    'hydra:writable': true,
                                    'hydra:description': 'The name of the item.'
                                }
                            ],
                            'hydra:supportedOperation': [
                                {
                                    '@type': 'hydra:Operation',
                                    'hydra:method': 'GET',
                                    'hydra:title': 'Retrieves Person resource.',
                                    'rdfs:label': 'Retrieves Person resource.',
                                    'returns': 'http://schema.org/Person'
                                },
                                {
                                    '@type': 'hydra:ReplaceResourceOperation',
                                    'expects': 'http://schema.org/Person',
                                    'hydra:method': 'PUT',
                                    'hydra:title': 'Replaces the Person resource.',
                                    'rdfs:label': 'Replaces the Person resource.',
                                    'returns': 'http://schema.org/Person'
                                },
                                {
                                    '@type': 'hydra:Operation',
                                    'hydra:method': 'DELETE',
                                    'hydra:title': 'Deletes the Person resource.',
                                    'rdfs:label': 'Deletes the Person resource.',
                                    'returns': 'owl:Nothing'
                                }
                            ]
                        },
                        {
                            '@id': '#Entrypoint',
                            '@type': 'hydra:Class',
                            'hydra:title': 'The API entrypoint',
                            'hydra:supportedProperty': [
                                {
                                    '@type': 'hydra:SupportedProperty',
                                    'hydra:property': {
                                        '@id': '#Entrypoint/blogPosting',
                                        '@type': 'hydra:Link',
                                        'domain': '#Entrypoint',
                                        'rdfs:label': 'The collection of BlogPosting resources',
                                        'range': 'hydra:PagedCollection',
                                        'hydra:supportedOperation': [
                                            {
                                                '@type': 'hydra:Operation',
                                                'hydra:method': 'GET',
                                                'hydra:title': 'Retrieves the collection of BlogPosting resources.',
                                                'rdfs:label': 'Retrieves the collection of BlogPosting resources.',
                                                'returns': 'hydra:PagedCollection'
                                            },
                                            {
                                                '@type': 'hydra:CreateResourceOperation',
                                                'expects': 'http://schema.org/BlogPosting',
                                                'hydra:method': 'POST',
                                                'hydra:title': 'Creates a BlogPosting resource.',
                                                'rdfs:label': 'Creates a BlogPosting resource.',
                                                'returns': 'http://schema.org/BlogPosting'
                                            }
                                        ]
                                    },
                                    'hydra:title': 'The collection of BlogPosting resources',
                                    'hydra:readable': true,
                                    'hydra:writable': false
                                },
                                {
                                    '@type': 'hydra:SupportedProperty',
                                    'hydra:property': {
                                        '@id': '#Entrypoint/person',
                                        '@type': 'hydra:Link',
                                        'domain': '#Entrypoint',
                                        'rdfs:label': 'The collection of Person resources',
                                        'range': 'hydra:PagedCollection',
                                        'hydra:supportedOperation': [
                                            {
                                                '@type': 'hydra:Operation',
                                                'hydra:method': 'GET',
                                                'hydra:title': 'Retrieves the collection of Person resources.',
                                                'rdfs:label': 'Retrieves the collection of Person resources.',
                                                'returns': 'hydra:PagedCollection'
                                            },
                                            {
                                                '@type': 'hydra:CreateResourceOperation',
                                                'expects': 'http://schema.org/Person',
                                                'hydra:method': 'POST',
                                                'hydra:title': 'Creates a Person resource.',
                                                'rdfs:label': 'Creates a Person resource.',
                                                'returns': 'http://schema.org/Person'
                                            }
                                        ]
                                    },
                                    'hydra:title': 'The collection of Person resources',
                                    'hydra:readable': true,
                                    'hydra:writable': false
                                }
                            ],
                            'hydra:supportedOperation': {
                                '@type': 'hydra:Operation',
                                'hydra:method': 'GET',
                                'rdfs:label': 'The API entrypoint.',
                                'returns': '#EntryPoint'
                            }
                        },
                        {
                            '@id': '#ConstraintViolation',
                            '@type': 'hydra:Class',
                            'hydra:title': 'A constraint violation',
                            'hydra:supportedProperty': [
                                {
                                    '@type': 'hydra:SupportedProperty',
                                    'hydra:property': {
                                        '@id': '#ConstraintViolation/propertyPath',
                                        '@type': 'rdf:Property',
                                        'rdfs:label': 'propertyPath',
                                        'domain': '#ConstraintViolation',
                                        'range': 'xmls:string'
                                    },
                                    'hydra:title': 'propertyPath',
                                    'hydra:description': 'The property path of the violation',
                                    'hydra:readable': true,
                                    'hydra:writable': false
                                },
                                {
                                    '@type': 'hydra:SupportedProperty',
                                    'hydra:property': {
                                        '@id': '#ConstraintViolation/message',
                                        '@type': 'rdf:Property',
                                        'rdfs:label': 'message',
                                        'domain': '#ConstraintViolation',
                                        'range': 'xmls:string'
                                    },
                                    'hydra:title': 'message',
                                    'hydra:description': 'The message associated with the violation',
                                    'hydra:readable': true,
                                    'hydra:writable': false
                                }
                            ]
                        },
                        {
                            '@id': '#ConstraintViolationList',
                            '@type': 'hydra:Class',
                            'subClassOf': 'hydra:Error',
                            'hydra:title': 'A constraint violation list',
                            'hydra:supportedProperty': [
                                {
                                    '@type': 'hydra:SupportedProperty',
                                    'hydra:property': {
                                        '@id': '#ConstraintViolationList/violation',
                                        '@type': 'rdf:Property',
                                        'rdfs:label': 'violation',
                                        'domain': '#ConstraintViolationList',
                                        'range': '#ConstraintViolation'
                                    },
                                    'hydra:title': 'violation',
                                    'hydra:description': 'The violations',
                                    'hydra:readable': true,
                                    'hydra:writable': false
                                }
                            ]
                        }
                    ]
                }];
            testing_1.expect(schemaHelper_1.schemaHelper.cleanModelsList(jsonApiPlatform[0]['hydra:supportedClass'])[0]['@id']).toContain('http://schema.org/Person');
        });
    });
});
//# sourceMappingURL=schemaHelper.spec.js.map