"use strict";
var testing_1 = require('@angular/core/testing');
var testing_2 = require('@angular/http/testing');
var core_1 = require('@angular/core');
var http_1 = require('@angular/http');
require('rxjs/add/operator/filter');
var Api_service_1 = require('./Api.service');
var Config_service_1 = require('../config/Config.service');
var schema_service_1 = require('../schema/schema.service');
var EntryPoint_1 = require('../models/EntryPoint');
var rxjs_1 = require('rxjs');
var CoreDefinition_service_1 = require('../core/CoreDefinition.service');
testing_1.describe('API service', function () {
    var apiService, configService, schemaService, mockBackend;
    testing_1.beforeEachProviders(function () {
        return [
            http_1.BaseRequestOptions,
            testing_2.MockBackend,
            Api_service_1.APIService,
            schema_service_1.SchemaService,
            core_1.provide(Config_service_1.ConfigService, {
                useValue: {
                    get: function () { return 'http://localhost:8000'; }
                }
            }),
            core_1.provide(CoreDefinition_service_1.CoreDefinitionService, {
                useValue: {
                    definitions: rxjs_1.Observable.of({
                        'firstPage': 'hydra:firstPage',
                        'lastPage': 'hydra:lastPage',
                        'itemsPerPage': 'hydra:itemsPerPage',
                        'totalItems': 'hydra:totalItems',
                        'member': 'hydra:member',
                        'toto': 'hydra:toto'
                    })
                }
            }),
            core_1.provide(http_1.Http, {
                useFactory: function (backend, defaultOptions) {
                    return new http_1.Http(backend, defaultOptions);
                }, deps: [testing_2.MockBackend, http_1.BaseRequestOptions]
            })
        ];
    });
    testing_1.beforeEach(testing_1.inject([Api_service_1.APIService, schema_service_1.SchemaService, Config_service_1.ConfigService, testing_2.MockBackend], testing_1.fakeAsync(function (ApiService, SchemaService, ConfigService, MockBackend) {
        apiService = ApiService;
        schemaService = SchemaService;
        configService = ConfigService;
        mockBackend = MockBackend;
    })));
    testing_1.describe('getCollectionByUrl', function () {
        testing_1.it('should load api definition', testing_1.inject([Api_service_1.APIService, testing_2.MockBackend], testing_1.fakeAsync(function (apiService, mockBackend) {
            mockBackend.connections.subscribe(function (c) {
                testing_1.expect(c.request.url).toBe('http://localhost:8000/people');
                var response = new http_1.ResponseOptions({
                    body: {
                        '@context': '/contexts/Person',
                        '@id': '/people',
                        '@type': 'hydra:PagedCollection',
                        'hydra:totalItems': 2,
                        'hydra:itemsPerPage': 30,
                        'hydra:firstPage': '/people',
                        'hydra:lastPage': '/people',
                        'hydra:member': [
                            {
                                '@id': '/people/1',
                                '@type': 'http://schema.org/Person',
                                'familyName': 'Doe'
                            },
                            {
                                '@id': '/people/2',
                                '@type': 'http://schema.org/Person',
                                'familyName': 'Norris'
                            }
                        ]
                    }
                });
                c.mockRespond(new http_1.Response(response));
            });
            var query = apiService.getCollectionByUrl('http://localhost:8000/people');
            testing_1.tick();
            query.subscribe(function (collection) {
                testing_1.expect(collection.members.length).toBe(2);
            });
        })));
        testing_1.it('should throw Error if API is unreachable', function () {
            mockBackend.connections.subscribe(function (c) {
                c.mockError();
            });
            testing_1.expect(function () { return apiService.getCollectionByUrl(''); }).toThrow();
        });
    });
    testing_1.describe('getCollection', function () {
        testing_1.beforeEach(testing_1.fakeAsync(testing_1.inject([Api_service_1.APIService], function (apiService) {
            spyOn(apiService, 'getCollectionByUrl').and.returnValue(rxjs_1.Observable.of({
                '@context': '/people',
            }));
            var entrypoint = new EntryPoint_1.EntryPoint('person');
            entrypoint.url = '/people';
            apiService.entrypoints = [
                entrypoint
            ];
        })));
    });
});
//# sourceMappingURL=Api.service.spec.js.map