"use strict";
var testing_1 = require('@angular/core/testing');
var Config_service_1 = require('./Config.service');
testing_1.describe('Config service', function () {
    testing_1.beforeEachProviders(function () { return Config_service_1.ConfigService; });
    testing_1.describe('setConfiguration', function () {
        testing_1.it('should be hookable with available values', testing_1.inject([Config_service_1.ConfigService], function (configService) {
            var defaultConfig = {
                api: {
                    baseUrl: 'http://localhost:8000'
                }
            };
            configService.setConfiguration(defaultConfig);
            testing_1.expect(configService.api.baseUrl).toBe('http://localhost:8000');
        }));
    });
    testing_1.describe('getRecursive', function () {
        var config;
        testing_1.beforeEach(function () {
            config = {
                api: {
                    baseUrl: 'http://localhost:8000'
                }
            };
        });
        testing_1.it('should find value by parsing configuration key by key', testing_1.inject([Config_service_1.ConfigService], function (configService) {
            testing_1.expect(configService.getRecursive(['api', 'baseUrl'], config)).toEqual('http://localhost:8000');
        }));
        testing_1.it('should send null when nothing is found', testing_1.inject([Config_service_1.ConfigService], function (configService) {
            testing_1.expect(configService.getRecursive(null, config)).toBe(null);
            testing_1.expect(configService.getRecursive([], config)).toBe(null);
            testing_1.expect(configService.getRecursive(['apis', 'baseUrl'], config)).toBe(null);
        }));
    });
    testing_1.describe('get', function () {
        testing_1.it('should ask for configuration parsing', testing_1.inject([Config_service_1.ConfigService], function (configService) {
            spyOn(configService, 'getRecursive').and.returnValue('http://localhost:8000');
            var result = configService.get('api.baseUrl');
            testing_1.expect(configService.getRecursive).toHaveBeenCalledWith(['api', 'baseUrl']);
            testing_1.expect(result).toEqual('http://localhost:8000');
        }));
        testing_1.it('should return null by default', testing_1.inject([Config_service_1.ConfigService], function (configService) {
            spyOn(configService, 'getRecursive');
            var result = configService.get();
            testing_1.expect(configService.getRecursive).not.toHaveBeenCalled();
            testing_1.expect(result).toBe(null);
        }));
    });
});
//# sourceMappingURL=Config.service.spec.js.map