"use strict";
var testing_1 = require('@angular/core/testing');
var http_1 = require('@angular/http');
var core_1 = require('@angular/core');
var testing_2 = require('@angular/http/testing');
var CoreDefinition_service_1 = require('./CoreDefinition.service');
var rxjs_1 = require('rxjs');
require('rxjs/add/observable/of');
var Config_service_1 = require('../config/Config.service');
testing_1.describe('Core Definition Service', function () {
    testing_1.beforeEachProviders(function () {
        return [
            CoreDefinition_service_1.CoreDefinitionService,
            core_1.provide(Config_service_1.ConfigService, {
                useValue: {
                    get: function () { return 'http://localhost:8000'; }
                }
            }),
        ];
    });
    testing_1.describe('loadDefinitions', function () {
        testing_1.it('should ask for definitions', testing_1.inject([CoreDefinition_service_1.CoreDefinitionService], function (cds) {
            spyOn(cds, 'getDefinitions').and.returnValue(rxjs_1.Observable.of({}));
            cds.loadDefinitions();
            testing_1.expect(cds.getDefinitions).toHaveBeenCalled();
        }));
        testing_1.it('should store definitions to local storage', testing_1.inject([CoreDefinition_service_1.CoreDefinitionService], function (cds) {
            spyOn(cds, 'getDefinitions').and.returnValue(rxjs_1.Observable.of({}));
            spyOn(localStorage, 'setItem');
            cds.loadDefinitions();
            testing_1.expect(localStorage.setItem).toHaveBeenCalledWith('definitions', '{}');
        }));
        testing_1.it('should dispatch definitions', testing_1.inject([CoreDefinition_service_1.CoreDefinitionService], function (cds) {
            spyOn(cds, 'getDefinitions').and.returnValue(rxjs_1.Observable.of({
                'required': 'hydra:required'
            }));
            cds.loadDefinitions();
            cds.definitions.subscribe(function (definitions) {
                testing_1.expect(definitions.required).toEqual('hydra:required');
            });
        }));
        testing_1.it('should accept string as definition', testing_1.inject([CoreDefinition_service_1.CoreDefinitionService], function (cds) {
            spyOn(cds, 'getDefinitions').and.returnValue(rxjs_1.Observable.of({
                'required': 'hydra:required'
            }));
            spyOn(cds, 'add');
            cds.loadDefinitions();
            testing_1.expect(cds.add).toHaveBeenCalledWith('required', 'hydra:required', {});
        }));
        testing_1.it('should accept object with @id as definition', testing_1.inject([CoreDefinition_service_1.CoreDefinitionService], function (cds) {
            spyOn(cds, 'getDefinitions').and.returnValue(rxjs_1.Observable.of({
                'property': {
                    '@id': 'hydra:property'
                }
            }));
            spyOn(cds, 'add');
            cds.loadDefinitions();
            testing_1.expect(cds.add).toHaveBeenCalledWith('property', 'hydra:property', {});
        }));
        testing_1.it('should reject object without @id as definition', testing_1.inject([CoreDefinition_service_1.CoreDefinitionService], function (cds) {
            spyOn(cds, 'getDefinitions').and.returnValue(rxjs_1.Observable.of({
                'cc:license': {
                    '@type': '@id'
                }
            }));
            spyOn(cds, 'add');
            cds.loadDefinitions();
            testing_1.expect(cds.add).not.toHaveBeenCalled();
        }));
    });
    testing_1.describe('getDefinitions', function () {
        testing_1.beforeEachProviders(function () {
            return [
                testing_2.MockBackend,
                http_1.BaseRequestOptions,
                core_1.provide(http_1.Http, {
                    useFactory: function (backend, defaultOptions) {
                        return new http_1.Http(backend, defaultOptions);
                    }, deps: [testing_2.MockBackend, http_1.BaseRequestOptions]
                })
            ];
        });
        testing_1.it('should dispatch local storage definitions', testing_1.inject([CoreDefinition_service_1.CoreDefinitionService], function (cds) {
            spyOn(localStorage, 'getItem').and.returnValue('{"property":"hydra:property"}');
            var obs = cds.getDefinitions();
            testing_1.expect(localStorage.getItem).toHaveBeenCalled();
            obs.subscribe(function (definitions) {
                testing_1.expect(definitions.property).toEqual('hydra:property');
            });
        }));
        testing_1.it('should dispatch context api definitions', testing_1.async(testing_1.inject([CoreDefinition_service_1.CoreDefinitionService, testing_2.MockBackend, Config_service_1.ConfigService], function (cds, api, config) {
            spyOn(localStorage, 'getItem').and.returnValue(null);
            api.connections.subscribe(function (c) {
                testing_1.expect(c.request.url).toBe('http://localhost:8000');
                var response = new http_1.ResponseOptions({
                    body: {
                        '@context': {
                            'property': 'hydra:property'
                        },
                        'fakeValue': 'hydra:fakeValue'
                    }
                });
                c.mockRespond(new http_1.Response(response));
            });
            var obs = cds.getDefinitions();
            obs.subscribe(function (definitions) {
                testing_1.expect(definitions.property).toEqual('hydra:property');
                testing_1.expect(definitions.fakeValue).toBeUndefined();
            });
        })));
    });
    testing_1.describe('add', function () {
        var definitions;
        testing_1.beforeEach(function () {
            definitions = {};
        });
        testing_1.it('should define an enumerable property', testing_1.inject([CoreDefinition_service_1.CoreDefinitionService], function (cds) {
            cds.add('property', 'hydra:property', definitions);
            testing_1.expect(definitions.property).toEqual('hydra:property');
            testing_1.expect(definitions.propertyIsEnumerable('property')).toBeTruthy();
        }));
        testing_1.it('should define an unwritable property', testing_1.inject([CoreDefinition_service_1.CoreDefinitionService], function (cds) {
            cds.add('property', 'hydra:property', definitions);
            testing_1.expect(function () {
                definitions.property = 'custom:property';
            }).toThrow();
        }));
    });
});
//# sourceMappingURL=CoreDefinition.service.spec.js.map