"use strict";
var testing_1 = require('@angular/core/testing');
var Model_1 = require('../models/Model');
testing_1.describe('Model model', function () {
    testing_1.describe('#constructor', function () {
        testing_1.it('should set title', function () {
            var model = new Model_1.Model({ 'hydra:title': 'Person' }, { 'title': 'hydra:title' });
            testing_1.expect(model.title).toBe('Person');
        });
        testing_1.it('should set description', function () {
            var model = new Model_1.Model({ 'hydra:description': 'A person.' }, { 'description': 'hydra:description' });
            testing_1.expect(model.description).toBe('A person.');
        });
        testing_1.it('should set properties', function () {
            var model = new Model_1.Model({ 'hydra:supportedProperty': [{
                        'hydra:title': 'firstName',
                        'hydra:property': { '@type': 'rdf:Property' },
                        'hydra:range': 'xmls:string',
                        'hydra:required': false,
                        'hydra:readable': false,
                        'hydra:writable': false
                    }] }, {
                'title': 'hydra:title',
                'property': 'hydra:property',
                'range': 'hydra:range',
                'required': 'hydra:required',
                'readable': 'hydra:readable',
                'writable': 'hydra:writable',
                'supportedProperty': 'hydra:supportedProperty'
            });
            testing_1.expect(model.properties.length).toBe(1);
        });
        testing_1.it('should set link', function () {
            var model = new Model_1.Model({ 'link': '/list/people' });
            testing_1.expect(model.link).toBe('/list/people');
        });
        testing_1.it('should have default values', function () {
            var model = new Model_1.Model();
            testing_1.expect(model.title).toBe('');
            testing_1.expect(model.description).toBe('');
            testing_1.expect(Array.isArray(model.properties)).toBeTruthy();
            testing_1.expect(model.properties.length).toBe(0);
            testing_1.expect(Array.isArray(model.link)).toBeTruthy();
            testing_1.expect(model.link.length).toBe(2);
            testing_1.expect(model.link).toContain('/list/');
        });
    });
    testing_1.describe('~getProperties', function () {
        testing_1.it('should populate properties array', function () {
            var model = new Model_1.Model(), result = model.getProperties([{
                    'hydra:title': 'firstName',
                    'hydra:property': { '@type': 'rdf:Property' },
                    'hydra:range': 'xmls:string',
                    'hydra:required': true,
                    'hydra:readable': true,
                    'hydra:writable': true
                }], {
                'title': 'hydra:title',
                'property': 'hydra:property',
                'range': 'hydra:range',
                'required': 'hydra:required',
                'readable': 'hydra:readable',
                'writable': 'hydra:writable'
            }), property = result[0];
            testing_1.expect(property.label).toBe('firstName');
            testing_1.expect(property.type).toBe('rdf:Property');
            testing_1.expect(property.range).toBe('xmls:string');
            testing_1.expect(property.required).toBeTruthy;
            testing_1.expect(property.readable).toBeTruthy;
            testing_1.expect(property.writable).toBeTruthy;
        });
    });
});
//# sourceMappingURL=Model.spec.js.map