"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var core_1 = require('@angular/core');
var http_1 = require('@angular/http');
var rxjs_1 = require('rxjs');
require('rxjs/add/operator/map');
require('rxjs/add/operator/switchMap');
require('rxjs/add/operator/share');
var Config_service_1 = require('../config/Config.service');
var entrypointHelper_1 = require('../../helper/entrypointHelper');
var schemaHelper_1 = require('../../helper/schemaHelper');
var EntryPoint_1 = require('../models/EntryPoint');
var stringHelper_1 = require('../../helper/stringHelper');
var CoreDefinition_service_1 = require('../core/CoreDefinition.service');
var SchemaService = (function () {
    function SchemaService(http, config, coreDefinitionService) {
        this.http = http;
        this.config = config;
        this.coreDefinitionService = coreDefinitionService;
        this.availableEntrypoints = [];
        this.schema = new rxjs_1.BehaviorSubject(null);
        this.entrypoints = new rxjs_1.BehaviorSubject([]);
    }
    SchemaService.prototype.load = function () {
        var _this = this;
        this.coreDefinitionService.getDefinitions().subscribe(function (definition) {
            _this.loadRoot();
            _this.loadDocumentation(definition);
        });
    };
    SchemaService.prototype.loadRoot = function () {
        var _this = this;
        var request = this.config.get('api.baseUrl');
        var entrypoints = this.http.get(request)
            .map(function (data) { return data.json(); })
            .map(entrypointHelper_1.entrypointHelper.filterEntryPoints);
        entrypoints.subscribe((function (entrypoints) {
            for (var key in entrypoints) {
                var cleanModel = stringHelper_1.stringHelper.toCamelCase(entrypointHelper_1.entrypointHelper.getModelById(key)), entrypoint = entrypointHelper_1.entrypointHelper.getEntryPointByModel(key, _this.availableEntrypoints);
                if (undefined === entrypoint) {
                    entrypoint = new EntryPoint_1.EntryPoint(cleanModel);
                    _this.availableEntrypoints.push(entrypoint);
                }
                entrypoint.url = entrypoints[key];
            }
            _this.entrypoints.next(_this.availableEntrypoints);
        }), function () {
            throw new Error("API's url is unreachable: " + request + ".");
        });
    };
    SchemaService.prototype.loadDocumentation = function (definition) {
        var request = [this.config.get('api.baseUrl'), 'vocab'].join('/'), apidoc = this.http.get(request)
            .map(function (data) { return data.json(); })
            .share();
        apidoc.subscribe(null, function () {
            throw new Error("API's url is unreachable: " + request + ".");
        });
        var api = apidoc.map(function (data) { return data[definition.supportedClass]; })
            .map(entrypointHelper_1.entrypointHelper.filterSupportedClass)
            .switchMap(function (results) { return results; });
        this.loadCollectionOperations(api, definition);
        this.loadSingleOperations(api, definition);
        this.loadSchema(apidoc, definition);
    };
    SchemaService.prototype.loadCollectionOperations = function (api, definition) {
        var _this = this;
        api.filter(function (item) { return item['@id'] === '#Entrypoint'; })
            .map(function (entrypoints) { return entrypoints[definition.supportedProperty]; })
            .switchMap(function (entrypoint) { return entrypoint; })
            .map(function (entrypoint) { return entrypoint[definition.property]; })
            .subscribe((function (entrypoint) {
            var cleanModel = stringHelper_1.stringHelper.toCamelCase(entrypointHelper_1.entrypointHelper.getModelById(entrypoint['@id'])), ep = entrypointHelper_1.entrypointHelper.getEntryPointByModel(cleanModel, _this.availableEntrypoints);
            if (undefined === ep) {
                ep = new EntryPoint_1.EntryPoint(cleanModel);
                _this.availableEntrypoints.push(ep);
            }
            var operations = entrypointHelper_1.entrypointHelper.getOperations(entrypoint[definition.supportedOperation], definition);
            ep.addOperations(operations);
            _this.entrypoints.next(_this.availableEntrypoints);
        }));
    };
    SchemaService.prototype.loadSingleOperations = function (api, definition) {
        var _this = this;
        api.filter(function (item) { return item['@id'] !== '#Entrypoint'; })
            .subscribe((function (entrypoint) {
            var cleanModel = stringHelper_1.stringHelper.toCamelCase(entrypointHelper_1.entrypointHelper.getModelById(entrypoint[definition.title])), ep = entrypointHelper_1.entrypointHelper.getEntryPointByModel(cleanModel, _this.availableEntrypoints), operations = entrypointHelper_1.entrypointHelper.getOperations(entrypoint[definition.supportedOperation], definition);
            if (undefined === ep) {
                ep = new EntryPoint_1.EntryPoint(cleanModel);
                _this.availableEntrypoints.push(ep);
            }
            ep.addOperations(operations);
            _this.entrypoints.next(_this.availableEntrypoints);
        }));
    };
    SchemaService.prototype.loadSchema = function (apidoc, definition) {
        var _this = this;
        apidoc
            .map(function (data) {
            return {
                title: schemaHelper_1.schemaHelper.getSchemaTitle(data, definition),
                models: schemaHelper_1.schemaHelper.populateModels(data, definition)
            };
        })
            .subscribe(function (schema) { return _this.schema.next(schema); });
    };
    SchemaService = __decorate([
        core_1.Injectable(), 
        __metadata('design:paramtypes', [http_1.Http, Config_service_1.ConfigService, CoreDefinition_service_1.CoreDefinitionService])
    ], SchemaService);
    return SchemaService;
}());
exports.SchemaService = SchemaService;
//# sourceMappingURL=schema.service.js.map