"use strict";
var testing_1 = require('@angular/core/testing');
var testing_2 = require('@angular/http/testing');
var core_1 = require('@angular/core');
var http_1 = require('@angular/http');
var Config_service_1 = require('../config/Config.service');
var schema_service_1 = require('./schema.service');
var rxjs_1 = require('rxjs');
require('rxjs/add/operator/filter');
var EntryPoint_1 = require('../models/EntryPoint');
testing_1.describe('Schema service', function () {
    testing_1.beforeEachProviders(function () {
        return [
            http_1.BaseRequestOptions,
            testing_2.MockBackend,
            schema_service_1.SchemaService,
            core_1.provide(Config_service_1.ConfigService, {
                useValue: {
                    get: function () { return 'http://localhost:8000'; }
                }
            }),
            core_1.provide(http_1.Http, {
                useFactory: function (backend, defaultOptions) {
                    return new http_1.Http(backend, defaultOptions);
                }, deps: [testing_2.MockBackend, http_1.BaseRequestOptions]
            })
        ];
    });
    testing_1.describe('#constructor', function () {
        testing_1.it('should init schema observable', testing_1.inject([schema_service_1.SchemaService], function (schemaService) {
            schemaService.schema.subscribe(function (schema) {
                testing_1.expect(schema).toBeNull();
            });
        }));
        testing_1.it('should init entrypoints observable', testing_1.inject([schema_service_1.SchemaService], function (schemaService) {
            schemaService.entrypoints.subscribe(function (schema) {
                testing_1.expect(Array.isArray(schema)).toBeTruthy();
                testing_1.expect(schema.length).toBe(0);
            });
        }));
    });
    testing_1.describe('load', function () {
        testing_1.it('should load api definition', testing_1.inject([schema_service_1.SchemaService], function (schemaService) {
            spyOn(schemaService, 'loadRoot');
            spyOn(schemaService, 'loadDocumentation');
            schemaService.load();
            testing_1.expect(schemaService.loadRoot).toHaveBeenCalled();
            testing_1.expect(schemaService.loadDocumentation).toHaveBeenCalled();
        }));
    });
    testing_1.describe('loadRoot', function () {
        testing_1.it('should load api definition', testing_1.inject([schema_service_1.SchemaService, testing_2.MockBackend], testing_1.fakeAsync(function (schemaService, mockBackend) {
            mockBackend.connections.subscribe(function (c) {
                testing_1.expect(c.request.url).toBe('http://localhost:8000');
                var response = new http_1.ResponseOptions({
                    body: {
                        '@context': '/contexts/Entrypoint',
                        '@id': '/',
                        '@type': 'Entrypoint',
                        'person': '/people',
                        'blogPosting': '/blog_postings'
                    }
                });
                c.mockRespond(new http_1.Response(response));
            });
            schemaService.entrypoints
                .filter(function (entrypoints) { return 0 < entrypoints.length; })
                .subscribe(function (entrypoints) {
                testing_1.expect(entrypoints.length).toBe(2);
                testing_1.expect(entrypoints[0] instanceof EntryPoint_1.EntryPoint).toBeTruthy();
            });
            schemaService.loadRoot();
        })));
        testing_1.it('should throw Error if API is unreachable', testing_1.inject([schema_service_1.SchemaService, testing_2.MockBackend], testing_1.fakeAsync(function (schemaService, mockBackend) {
            mockBackend.connections.subscribe(function (c) {
                c.mockError();
            });
            testing_1.expect(function () { return schemaService.loadRoot(); }).toThrow();
        })));
    });
    testing_1.describe('loadDocumentation', function () {
        testing_1.it('should load operations and schema', testing_1.inject([schema_service_1.SchemaService, testing_2.MockBackend], testing_1.fakeAsync(function (schemaService, mockBackend) {
            mockBackend.connections.subscribe(function (c) {
                testing_1.expect(c.request.url).toBe('http://localhost:8000/vocab');
                var response = new http_1.ResponseOptions({
                    body: {
                        'hydra:title': 'Hydra API Test',
                        'hydra:supportedClass': [
                            { '@id': 'http://schema.org/Person' },
                            { '@id': 'http://schema.org/BlogPosting' },
                            { '@id': '#Entrypoint' },
                            { '@id': '#ConstraintViolation' },
                            { '@id': '#ConstraintViolationList' }
                        ]
                    }
                });
                c.mockRespond(new http_1.Response(response));
            });
            spyOn(schemaService, 'loadCollectionOperations');
            spyOn(schemaService, 'loadSingleOperations');
            spyOn(schemaService, 'loadSchema');
            schemaService.loadDocumentation({
                'title': 'hydra:title',
                'supportedClass': 'hydra:supportedclass'
            });
            testing_1.tick();
            testing_1.expect(schemaService.loadCollectionOperations).toHaveBeenCalled();
            testing_1.expect(schemaService.loadSingleOperations).toHaveBeenCalled();
            testing_1.expect(schemaService.loadSchema).toHaveBeenCalled();
        })));
        testing_1.it('should throw Error if API is unreachable', testing_1.inject([schema_service_1.SchemaService, testing_2.MockBackend], testing_1.fakeAsync(function (schemaService, mockBackend) {
            mockBackend.connections.subscribe(function (c) {
                c.mockError();
            });
            testing_1.expect(function () { return schemaService.loadDocumentation({
                'title': 'hydra:title',
                'supportedClass': 'hydra:supportedclass'
            }); }).toThrow();
        })));
    });
    testing_1.describe('loadCollectionOperations', function () {
        testing_1.it('should load collection operations for each entrypoint', testing_1.inject([schema_service_1.SchemaService], function (schemaService) {
            var index = 0;
            schemaService.entrypoints
                .filter(function (entrypoints) { return 0 < entrypoints.length; })
                .subscribe(function (entrypoints) {
                if (2 === ++index) {
                    testing_1.expect(entrypoints.length).toBe(2);
                    testing_1.expect(entrypoints[0] instanceof EntryPoint_1.EntryPoint).toBeTruthy();
                    testing_1.expect(entrypoints[0].operations).toContain('GET');
                    testing_1.expect(entrypoints[0].operations).not.toContain('POST');
                    testing_1.expect(entrypoints[1] instanceof EntryPoint_1.EntryPoint).toBeTruthy();
                    testing_1.expect(entrypoints[1].operations).toContain('GET');
                    testing_1.expect(entrypoints[1].operations).toContain('POST');
                }
            });
            var observable = new rxjs_1.BehaviorSubject({
                '@id': '#Entrypoint',
                'hydra:supportedProperty': [
                    {
                        'hydra:property': {
                            '@id': '#Entrypoint/person',
                            'hydra:supportedOperation': [{ 'hydra:method': 'GET' }]
                        }
                    },
                    {
                        'hydra:property': {
                            '@id': '#Entrypoint/BlogPosting',
                            'hydra:supportedOperation': [{ 'hydra:method': 'GET' }, { 'hydra:method': 'POST' }]
                        }
                    }
                ]
            });
            schemaService.loadCollectionOperations(observable, {
                'property': 'hydra:property',
                'supportedProperty': 'hydra:supportedProperty',
                'supportedOperation': 'hydra:supportedOperation',
                'PagedCollection': 'hydra:PagedCollection',
                'method': 'hydra:method',
                'title': 'hydra:title'
            });
        }));
    });
    testing_1.describe('loadSingleOperations', function () {
        testing_1.it('should load single operations for each entrypoint', testing_1.async(testing_1.inject([schema_service_1.SchemaService], function (schemaService) {
            schemaService.entrypoints
                .filter(function (entrypoints) { return 0 < entrypoints.length; })
                .subscribe(function (entrypoints) {
                testing_1.expect(entrypoints.length).toBe(1);
                testing_1.expect(entrypoints[0] instanceof EntryPoint_1.EntryPoint).toBeTruthy();
                testing_1.expect(entrypoints[0].operations).toContain('GET');
                testing_1.expect(entrypoints[0].operations).toContain('POST');
                testing_1.expect(entrypoints[0].operations).toContain('DELETE');
            });
            var observable = new rxjs_1.BehaviorSubject({
                'hydra:title': 'Person',
                'hydra:supportedOperation': [
                    { 'hydra:method': 'GET' },
                    { 'hydra:method': 'POST' },
                    { 'hydra:method': 'DELETE' }
                ]
            });
            schemaService.loadSingleOperations(observable, {
                'title': 'hydra:title',
                'supportedOperation': 'hydra:supportedOperation',
                'PagedCollection': 'hydra:PagedCollection',
                'method': 'hydra:method'
            });
        })));
    });
    testing_1.describe('loadSchema', function () {
        testing_1.it('should load title and models', testing_1.async(testing_1.inject([schema_service_1.SchemaService], function (schemaService) {
            schemaService.schema
                .filter(function (schema) { return null !== schema; })
                .subscribe(function (schema) {
                testing_1.expect(schema.title).toBeUndefined();
                testing_1.expect(schema.models).toBeDefined();
                testing_1.expect(Array.isArray(schema.models)).toBeTruthy();
            });
            var observable = new rxjs_1.BehaviorSubject({});
            schemaService.loadSchema(observable, {
                'title': 'hydra:title',
                'supportedOperation': 'hydra:supportedOperation',
                'PagedCollection': 'hydra:PagedCollection',
                'method': 'hydra:method'
            });
        })));
    });
});
//# sourceMappingURL=schema.service.spec.js.map